#include "pathfinder.h"

#include <map>
#include <queue>
#include <algorithm>
//#include "fiboheap.h"

PathFinder::PathFinder(){
}

std::vector<GraphNode*> PathFinder::a_star(GraphNode* start,GraphNode* goal)
{
    // Check if priorityqueue sort value in right order.
    std::priority_queue<PriorityNode*> frontier = std::priority_queue<PriorityNode*>();
    std::map<GraphNode*, float> cost = std::map<GraphNode*, float>(); //cost of visited node
    std::map<GraphNode*, GraphNode*> pred = std::map<GraphNode*, GraphNode*>(); //pred of visited node

    // init frontier, cost, and pred with value for start
    PriorityNode* pn = new PriorityNode();
    pn->node = std::pair<GraphNode*,float>(start,0);
    frontier.push(pn);
    cost.insert(std::pair<GraphNode*,float>(start,0));
    pred.insert(std::pair<GraphNode*,GraphNode*>(start,NULL));

    GraphNode* current;
    while(!frontier.empty()){
        //pick best element from frontier (with priority queue the best is in front)
        pn = frontier.top();
        current = pn->node.first;
        frontier.pop();

        // goal reached, end of a-star
        if (current == goal){
            break;
        }

        // for all neighbours of current node
        for (GraphEdge next : current->getNeighbours()){
             float new_cost = cost[current] + next.getWeight();

             if ((cost.count(next.getTarget()) == 0) || (new_cost < cost[next.getTarget()])){
                // affect processed cost to next node in cost_map
                cost[next.getTarget()] = new_cost;

                // calcul priority of node with heuristic,and add it to frontier                
                float priority = new_cost; //+ heuristic(next.getTarget(), goal);
                PriorityNode* pn = new PriorityNode();
                pn->node = std::pair<GraphNode*,float>(next.getTarget(),priority);
                frontier.push(pn);

                // memorize predecessor for next
                 pred[next.getTarget()] = current;
             }
        }
    }

    // reconstruct path by backtracking from goal to start
    std::vector<GraphNode*> path = std::vector<GraphNode*>();
    while(current != start){
        path.push_back(current);
        current = pred[current];
    }
    path.push_back(start);
    std::reverse(path.begin(),path.end());
    return path;
}


